%WRITESDML Writes data in StatDataML-format.
%
% [S = ] WRITESDML(X, FILENAME, {'OPTION 1',OPTION1,} ...)
%
% X allows for any data-object.
% If FILENAME is ommited, the standard-output is used.
%
% There are several options set to the
% indicated default values if ommited:
%
% TEXTDATA='FALSE': uses the more compact "textdata"-store mode;
% default is false because this feature is not necesserely
% implemented in all StatDataML-modules and hence, it cannot be
% garanteed that the produced StatDataML-file will be read correctly.
%
% The following parameters are used in the TEXTDATA-mode:
%
% SEP=' &#x000A;&#x000D;': String containing the separator-characters (only the first
% character is used in WRITESDML). If data is of mode charcter, this
% string has usually to be changed to characters not contained in the data.
%
% NULL='NULL': String representing an empty string. This string should
% also not occur in the data. Used for character data only.
%
% NAN='nan': String representing a NaN ('Not a Number'). This string should
% also not occur in the data.
%
% POSINF='inf': String representing +Inf. This string should
% also not occur in the data.
%
% NEGINF='-inf': String representing -Inf. This string should
% also not occur in the data.
% TRUE, FALSE: String representing true and false values for
% logical data.
%
% DROP-DIM=true: drops all singleton-dimensions.
%
% DTD='StatDataML.dtd': name and partial-path of the StatDataML-data-
% validation-file, allowing syntactical checking when the
% file is read by an XML-parser. The complete path has to be build
% by the reading appliction.
%
% DTD_DEF='StatDataML.dtd': URL used, if the file cannot be found with DTD
%
% TITLE='MatLab-Dataset': describes data-content.
%
% SOURCE='MatLab': data-source.
%
% VERSION=' ': data-version.
%
% COMMENT=' ': any commentary.
%
% DATE=now: date and time of data-export
% (default: system data).
%
% AS.STRING='FALSE': if `TRUE', the output is written to `S'
% returned by the function.
%
% See also READSDML.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% writeSDML for MatLab/Octave
%%
%% exports data in StatDataML-format
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% function writesdml (x; filename:string):
%
% Main function; creates header and calls "check"
%

function S = writesdml(x, filename, varargin)

% Parameters - set defaults
global DROP_DIM; DROP_DIM = 1;
DTD = 'StatDataML.dtd';
DTD_DEF = 'StatDataML.dtd';
PKG = 'StatDataML_1.0-0';



TITLE = 'octave-data';

if isempty (TITLE) TITLE = ' '; end



SOURCE = 'octave';

VERSION = ' ';
COMMENT = ' ';
global ASSTRING; ASSTRING = 0;
global TEXTDATA; TEXTDATA = 0;



global SEP; SEP = ' &#x000A;&#x000D;';

global NULL; NULL = 'NULL';
global NAN; NAN = 'nan';
global POSINF; POSINF = 'inf';
global NEGINF; NEGINF = '-inf';
global TRUE; TRUE = '1';
global FALSE; FALSE = '0';

global STR; STR = '';

DATE=sprintf('%0 4u-%0 2u-%0 2uT%0 2u:%0 2u:%0 2u', fix(clock));

% Parameters - scan arguments:

if sdml_smod (nargin,2) > 0 % equal number of arguments required
        disp ('missing argument !');
        return;
end

for i=1:2:(nargin-2)




        c = upper(va_arg());
        s = va_arg();

        if isempty(s) s=' '; end
        if strcmp ('DROP_DIM',c) DROP_DIM = s;
        elseif strcmp ('TEXTDATA',c) TEXTDATA = strcmp(upper(s),'TRUE');
        elseif strcmp ('SEP',c) SEP = s;
        elseif strcmp ('DTD',c) DTD = s;
        elseif strcmp ('DTD_DEF',c) DTD_DEF = s;
        elseif strcmp ('NULL',c) NULL = s;
        elseif strcmp ('NAN',c) NAN = s;
        elseif strcmp ('POSINF',c) POSINF = s;
        elseif strcmp ('NEGINF',c) NEGINF = s;
        elseif strcmp ('TRUE',c) TRUE = s;
        elseif strcmp ('FALSE',c) FALSE = s;
        elseif strcmp ('PKG',c) PKG = s;
        elseif strcmp ('TITLE',c) TITLE = s;
        elseif strcmp ('SOURCE',c) SOURCE = s;
        elseif strcmp ('VERSION',c) VERSION = s;
        elseif strcmp ('COMMENT',c) COMMENT = s;
        elseif strcmp ('DATE',c) DATE = s;
        elseif strcmp ('AS.STRING',c) ASSTRING = strcmp(upper(s),'TRUE');
        else
                disp (strcat('ERROR-->',c,': unknown argument.'));
                return;
        end;


end

% open output file - if ommited, use std-out
global fh;
if isempty(filename)
        fh=1; % std-out
else
        fh=fopen (filename, 'w');
end;

% Header
sdml_p('<?xml version="1.0"?>')
sdml_p(strcat('<!DOCTYPE StatDataML PUBLIC "',DTD,'" "',DTD_DEF,'">'))
sdml_p('<StatDataML xmlns="http://www.omegahat.org/StatDataML/">')

% Description

sdml_p('<description>')

sdml_p(strcat('<title>',TITLE,'</title>'))
sdml_p(strcat('<source>',SOURCE,'</source>'))
sdml_p(strcat('<date>',DATE,'</date>'))
sdml_p(strcat('<version>',VERSION,'</version>'))
sdml_p(strcat('<comment>',COMMENT,'</comment>'))



sdml_p(strcat('<creator>octave-',version,':',PKG,'</creator>'))


sdml_p('</description>')

% Data
sdml_p('<dataset>')
sdml_check(x)
sdml_p('</dataset>')

sdml_p('</StatDataML>')

if fh > 2 fclose (fh); end;

if ASSTRING == 1 S = STR; end;
